#-----------------------------------------------------------------------
# A serie of basic commands of the GILDAS-Python binding callable from
# Python.
#
# S.Bardeau, 11-sep-2019 ( gildas _at_ iram.fr )
#-----------------------------------------------------------------------

### Python modules ###

import pysic

pysic

# List the content of attributes and methods. See full
# documentation for details.
dir(pysic)

# .comm() method send a single command line to the SIC interpretor
print pysic.comm.__doc__
pysic.comm('EXA PI')

# .enter() starts the SIC prompt
pysic.enter()
# go back with the SIC\PYTHON command
SIC\PYTHON


### Sharing variables ###

# The dictionary of shared variables:
# .exa() method lists the shared variables
pysic.exa()
# You can see they are not in your name area
dir()

# Actually you access the shared variables through the .gdict attribute:
dir(pysic)
print pysic.gdict.__doc__
pysic.gdict.pi

# You can create a shorcut (a new 'reference'):
d = pysic.gdict
d.pi


# Defining variables: usual way

pysic.comm('DEFINE INTEGER A')
d.a
pysic.comm('LET A 123')
d.a
pysic.comm('DELETE /VARIABLE A')
d.a  # Returns an error since A does not exist anymore


# Defining variables: nice way

d.b = 456  # Scalar
pysic.comm('EXA B')
del d.b
pysic.comm('EXA B')

d.c = (1.23,4.56,7.89)  # Array
pysic.comm('EXA C')
d.d = ( ("ABC","DEF") , ("GHI","JKL") )
pysic.comm('EXA D')
del d.c,d.d


# Illegal variable name
pysic.comm('DEFINE REAL IMPORT')
import = 1  # Reserved keyword: this line is an error for Python
# Variable exists on Gildas side but was not imported in Python
pysic.comm('EXAMINE IMPORT')
pysic.comm('DELETE /VAR IMPORT')


### General array handling ###

# The shared arrays benefit the numpy-arrays behaviors:
d.a = (0,0,0)
d.a[0]   # Elemental access
d.a[1:]  # Sub-arrays. Remember thats indices are in range 0 : N-1
d.a += 1 # You can use advanced syntax like 'inplace addition'
d.a  # As seen from Python side
pysic.comm('EXAMINE A')  # As seen from Gildas side


# Sub-arrays:
b = d.a[0:2]  # Sub-array: 1st bound included, last bound excluded
b
# At this stage 'b' and 'a' share their elements. This is a
# numpy standard feature
print b  # 'b' is a specific object
b += 1  # Modifying b...
b
d.a     # ... modifies a!


# Structures:
pysic.define('structure e')
d.e
print d.e

# Structures sub-elements are accessed as attributes of the parent structure
# Attaching objects to a structure is as easy as defining a variable:
d.e.a = 1
d.e.b = (2.34,5.67)
pysic.comm("EXA E%")


# Nested structures
pysic.define('structure E%C')
d.e.c
d.e.c.a = 2
pysic.comm("EXA E%C%")


# Images:
pysic.define('image F centaurus.gdf WRITE')

# Images behave as arrays...
d.f
d.f[0,0]

# ... but also as structures containing the header elements
print d.f
d.f.sigma


# Read-only variables:
d.pi
d.pi = 0  # Error: can not overwrite


# Accessing variables in interactive mode
dir()  # Almost nothing in the current name area
# You can redefine the .gdict dictionary such as it is the current
# name area. 'globals()' function returns this name area
pysic.setgdict(globals())
dir()  # Now it is filled with the Gildas-Python shared variables
pysic.define("REAL R")
r
# The counterpart is that defining a variable in Gildas automatically
# imports it in your Python name area: this may overwrite your other
# variables, introduce conflict... Use with caution.
